/*
  Nokia 6100 Display Test
  Copyright 2006 Refik Hadzialic (http://www.e-dsp.com) but this code
  was derived from Thomas Pfeifer's code at
  http://thomaspfeifer.net/nokia_6100_display.htm and Owen Osborn's code
  at http://www.sparkfun.com/datasheets/LCD/Nokia6100_Demo.c. I don't 
  the credits for it. 


  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

/****************************************************/
/* Example Program For ET-LCD6610                   */
/* MCU      : ATMEGA16                              */
/*          : Frequency Bus = 16 MHz                */
/* Compiler : WinAVR-20071221+AVR Studio 4.13 SP2   */
/* Write By : ADISAK CHOOCHAN(ETT CO.,LTD.)         */
/* Mods.    : Piotr Rzeszut (piotrva@wp.pl)         */
/* Function : Demo Interface ET-LCD6610             */ 
/*          : Use Pin I/O Generate SPI Signal       */
/****************************************************/
/* Interface ET-LCD6610 By ATMEGA16                 */
/*    ATMEGA16 --> ET-LCD6610                       */
/*    PB4(I/O) --> #CS                              */
/*    PB7(I/O) --> SCLK                             */
/*    PB5(I/O) --> SDATA                            */
/*    PB0(I/O) --> #RESEET                          */
/*    +5V      --> BL,VCC,VBL                       */
/*    GND      --> GND                              */
/****************************************************/

#include <string.h>
#include <avr/io.h>
#include <util/delay.h>
#include <avr/pgmspace.h>

// Define Software SPI Pin Signal
#define SPIPORT PORTA
#define SPIDDR DDRA
#define CS 0             // PORTB.4 --> #CS
#define CLK 1            // PORTB.7 --> SCLK
#define SDA 2            // PORTB.5 --> SDATA
#define RESET 3          // PORTB.0 --> #RESET


// Epson S1D15G10 Command Set 
#define DISON       0xaf
#define DISOFF      0xae
#define DISNOR      0xa6
#define DISINV      0xa7
#define COMSCN      0xbb
#define DISCTL      0xca
#define SLPIN       0x95
#define SLPOUT      0x94
#define PASET       0x75
#define CASET       0x15
#define DATCTL      0xbc
#define RGBSET8     0xce
#define RAMWR       0x5c
#define RAMRD       0x5d
#define PTLIN       0xa8
#define PTLOUT      0xa9
#define RMWIN       0xe0
#define RMWOUT      0xee
#define ASCSET      0xaa
#define SCSTART     0xab
#define OSCON       0xd1
#define OSCOFF      0xd2
#define PWRCTR      0x20
#define VOLCTR      0x81
#define VOLUP       0xd6
#define VOLDOWN     0xd7
#define TMPGRD      0x82
#define EPCTIN      0xcd
#define EPCOUT      0xcc
#define EPMWR       0xfc
#define EPMRD       0xfd
#define EPSRRD1     0x7c
#define EPSRRD2     0x7d
#define NOP         0x25

//a few basic colors
#define RED				0xE0
#define GREEN			0x1C
#define BLUE			0x03
#define YELLOW			0xFC
#define MAGENTA			0xE3
#define CYAN			0x1F
#define BLACK			0x00
#define WHITE			0xFF

#define cbi(reg, bit) (reg&=~(1<<bit))
#define sbi(reg, bit) (reg|= (1<<bit))

#define CS0 cbi(SPIPORT,CS);
#define CS1 sbi(SPIPORT,CS);
#define CLK0 cbi(SPIPORT,CLK);
#define CLK1 sbi(SPIPORT,CLK);
#define SDA0 cbi(SPIPORT,SDA);
#define SDA1 sbi(SPIPORT,SDA);
#define RESET0 cbi(SPIPORT,RESET);
#define RESET1 sbi(SPIPORT,RESET);

#define byte unsigned char

void sendCMD(byte cmd);
void sendData(byte cmd);
void shiftBits(byte b);
void LCD_put_pixel(unsigned char color, unsigned char x, unsigned char y);


byte font[475] PROGMEM = {
0x00 , 0x00 , 0x00 , 0x00 , 0x00 ,
0x00 , 0x06 , 0x5F , 0x06 , 0x00 ,                      // !
0x07 , 0x03 , 0x00 , 0x07 , 0x03 ,                      // ,
0x24 , 0x7E , 0x24 , 0x7E , 0x24 ,                      // #
0x24 , 0x2B , 0x6A , 0x12 , 0x00 ,                      // $
0x63 , 0x13 , 0x08 , 0x64 , 0x63 ,                      // %
0x36 , 0x49 , 0x56 , 0x20 , 0x50 ,                      // &
0x00 , 0x07 , 0x03 , 0x00 , 0x00 ,                      // //
0x00 , 0x3E , 0x41 , 0x00 , 0x00 ,                      // (
0x00 , 0x41 , 0x3E , 0x00 , 0x00 ,                      // )
0x08 , 0x3E , 0x1C , 0x3E , 0x08 ,                      // *
0x08 , 0x08 , 0x3E , 0x08 , 0x08 ,                      // +
0x00 , 0xE0 , 0x60 , 0x00 , 0x00 ,                      // ,
0x08 , 0x08 , 0x08 , 0x08 , 0x08 ,                      // -
0x00 , 0x60 , 0x60 , 0x00 , 0x00 ,                      // .
0x20 , 0x10 , 0x08 , 0x04 , 0x02 ,                      // /
0x3E , 0x51 , 0x49 , 0x45 , 0x3E ,                      // 0
0x00 , 0x42 , 0x7F , 0x40 , 0x00 ,                      // 1
0x62 , 0x51 , 0x49 , 0x49 , 0x46 ,                      // 2
0x22 , 0x49 , 0x49 , 0x49 , 0x36 ,                      // 3
0x18 , 0x14 , 0x12 , 0x7F , 0x10 ,                      // 4
0x2F , 0x49 , 0x49 , 0x49 , 0x31 ,                      // 5
0x3C , 0x4A , 0x49 , 0x49 , 0x30 ,                      // 6
0x01 , 0x71 , 0x09 , 0x05 , 0x03 ,                      // 7
0x36 , 0x49 , 0x49 , 0x49 , 0x36 ,                      // 8
0x06 , 0x49 , 0x49 , 0x29 , 0x1E ,                      // 9
0x00 , 0x6C , 0x6C , 0x00 , 0x00 ,                      // :
0x00 , 0xEC , 0x6C , 0x00 , 0x00 ,                      // ;
0x08 , 0x14 , 0x22 , 0x41 , 0x00 ,                      // <
0x24 , 0x24 , 0x24 , 0x24 , 0x24 ,                      // =
0x00 , 0x41 , 0x22 , 0x14 , 0x08 ,                      // >
0x02 , 0x01 , 0x59 , 0x09 , 0x06 ,                      // ?
0x3E , 0x41 , 0x5D , 0x55 , 0x1E ,                      // @
0x7E , 0x09 , 0x09 , 0x09 , 0x7E ,                      // A
0x7F , 0x49 , 0x49 , 0x49 , 0x36 ,                      // B
0x3E , 0x41 , 0x41 , 0x41 , 0x22 ,                      // C
0x7F , 0x41 , 0x41 , 0x41 , 0x3E ,                      // D
0x7F , 0x49 , 0x49 , 0x49 , 0x41 ,                      // E
0x7F , 0x09 , 0x09 , 0x09 , 0x01 ,                      // F
0x3E , 0x41 , 0x49 , 0x49 , 0x7A ,                      // G
0x7F , 0x08 , 0x08 , 0x08 , 0x7F ,                      // H
0x00 , 0x41 , 0x7F , 0x41 , 0x00 ,                      // I
0x30 , 0x40 , 0x40 , 0x40 , 0x3F ,                      // J
0x7F , 0x08 , 0x14 , 0x22 , 0x41 ,                      // K
0x7F , 0x40 , 0x40 , 0x40 , 0x40 ,                      // L
0x7F , 0x02 , 0x04 , 0x02 , 0x7F ,                      // M
0x7F , 0x02 , 0x04 , 0x08 , 0x7F ,                      // N
0x3E , 0x41 , 0x41 , 0x41 , 0x3E ,                      // O
0x7F , 0x09 , 0x09 , 0x09 , 0x06 ,                      // P
0x3E , 0x41 , 0x51 , 0x21 , 0x5E ,                      // Q
0x7F , 0x09 , 0x09 , 0x19 , 0x66 ,                      // R
0x26 , 0x49 , 0x49 , 0x49 , 0x32 ,                      // S
0x01 , 0x01 , 0x7F , 0x01 , 0x01 ,                      // T
0x3F , 0x40 , 0x40 , 0x40 , 0x3F ,                      // U
0x1F , 0x20 , 0x40 , 0x20 , 0x1F ,                      // V
0x3F , 0x40 , 0x3C , 0x40 , 0x3F ,                      // W
0x63 , 0x14 , 0x08 , 0x14 , 0x63 ,                      // X
0x07 , 0x08 , 0x70 , 0x08 , 0x07 ,                      // Y
0x71 , 0x49 , 0x45 , 0x43 , 0x00 ,                      // Z
0x00 , 0x7F , 0x41 , 0x41 , 0x00 ,                      // [
0x02 , 0x04 , 0x08 , 0x10 , 0x20 ,                      // Back slash
0x00 , 0x41 , 0x41 , 0x7F , 0x00 ,                      // ]
0x04 , 0x02 , 0x01 , 0x02 , 0x04 ,                      // ^
0x80 , 0x80 , 0x80 , 0x80 , 0x80 ,                      // _
0x00 , 0x03 , 0x07 , 0x00 , 0x00 ,                      // `
0x20 , 0x54 , 0x54 , 0x54 , 0x78 ,                      // a
0x7F , 0x44 , 0x44 , 0x44 , 0x38 ,                      // b
0x38 , 0x44 , 0x44 , 0x44 , 0x28 ,                      // c
0x38 , 0x44 , 0x44 , 0x44 , 0x7F ,                      // d
0x38 , 0x54 , 0x54 , 0x54 , 0x18 ,                      // e
0x08 , 0x7E , 0x09 , 0x09 , 0x00 ,                      // f
0x18 , 0xA4 , 0xA4 , 0xA4 , 0x7C ,                      // g
0x7F , 0x04 , 0x04 , 0x78 , 0x00 ,                      // h
0x00 , 0x00 , 0x7D , 0x00 , 0x00 ,                      // i
0x40 , 0x80 , 0x84 , 0x7D , 0x00 ,                      // j
0x7F , 0x10 , 0x28 , 0x44 , 0x00 ,                      // k
0x00 , 0x00 , 0x7F , 0x40 , 0x00 ,                      // l
0x7C , 0x04 , 0x18 , 0x04 , 0x78 ,                      // m
0x7C , 0x04 , 0x04 , 0x78 , 0x00 ,                      // n
0x38 , 0x44 , 0x44 , 0x44 , 0x38 ,                      // o
0xFC , 0x44 , 0x44 , 0x44 , 0x38 ,                      // p
0x38 , 0x44 , 0x44 , 0x44 , 0xFC ,                      // q
0x44 , 0x78 , 0x44 , 0x04 , 0x08 ,                      // r
0x08 , 0x54 , 0x54 , 0x54 , 0x20 ,                      // s
0x04 , 0x3E , 0x44 , 0x24 , 0x00 ,                      // t
0x3C , 0x40 , 0x20 , 0x7C , 0x00 ,                      // u
0x1C , 0x20 , 0x40 , 0x20 , 0x1C ,                      // v
0x3C , 0x60 , 0x30 , 0x60 , 0x3C ,                      // w
0x6C , 0x10 , 0x10 , 0x6C , 0x00 ,                      // x
0x9C , 0xA0 , 0x60 , 0x3C , 0x00 ,                      // y
0x64 , 0x54 , 0x54 , 0x4C , 0x00 ,                      // z
0x08 , 0x3E , 0x41 , 0x41 , 0x00 ,                      // {
0x00 , 0x00 , 0x7F , 0x00 , 0x00 ,                      // |
0x00 , 0x41 , 0x41 , 0x3E , 0x08 ,                      // }
0x02 , 0x01 , 0x02 , 0x01 , 0x00                        // ~
};

void delay_ms(int ms) {
  int i;
  for (i=0;i<ms;i++) _delay_ms(1);
}


void shiftBits(byte b) {

  uint8_t mask=0b10000000;
	while(mask){
		CLK0
		if(b&mask) SDA1 else SDA0
		CLK1
		mask>>=1;
	}

}

void LCD_put_pixel(unsigned char color, unsigned char x, unsigned char y){
  sendCMD(CASET);   // page start/end ram
  sendData(x);      // for some reason starts at 2
  sendData(x+1);

  sendCMD(PASET);   // column start/end ram
  sendData(y);
  sendData(y+1);
  sendCMD(RAMWR);
  sendData(color);
}

inline void LCD_Pixel(unsigned char x,unsigned char y,unsigned char c){
	LCD_put_pixel(c,x,y);
}

//send data
void sendData(unsigned char data) {
  CLK1
  CS1
  CS0

  CLK0
  SDA1                                                 //1 for param
  CLK1

  shiftBits(data);
}

//send cmd
void sendCMD(unsigned char data) {

  CLK1
  CS1
  CS0

  CLK0
  SDA0
  CLK1

  shiftBits(data);

}

//  (x1,y1)
//     ***************
//     *             *
//     *             *
//     ***************
//                    (x2,y2)

void lcd_set_box(unsigned char x1, unsigned char y1, unsigned char x2, unsigned char y2)
{
  sendCMD(CASET);   // page start/end ram
  sendData(x1);     // for some reason starts at 2
  sendData(x2);

  sendCMD(PASET);   // column start/end ram
  sendData(y1);
  sendData(y2);
}

void lcd_clear(unsigned char color, unsigned char x1, unsigned char y1, unsigned char x2, unsigned char y2)
{
  uint16_t i;
  uint16_t total_bytes1;
  uint16_t total_bytes2;
  uint16_t total_bytes;

  lcd_set_box(x1, y1, x2, y2);

  sendCMD(RAMWR);
  total_bytes1 = (x2 - x1 + 1); 
  total_bytes2 = (y2 - y1 + 1);
  total_bytes = total_bytes1 * total_bytes2;
  for (i = 0; i < total_bytes; i++){
    sendData(color);
  }
}

void lcd_init()
{

	SPIDDR=(1<<SDA)|(1<<CLK)|(1<<CS)|(1<<RESET); //Port-Direction Setup
  // Hardware Reset LCD
  RESET0
  delay_ms(10);
  RESET1

  // Initial state
  CS1
  SDA1
  CLK1

  // Display control
  sendCMD(DISCTL);
  sendData(0b00001100); 		// 2 divisions, Field swithcing period
  sendData(32);   		// 132 lines to be display
  sendData(0b00011111);   		// Inversely hightlighted lines - 1
  sendData(0);

  sendCMD(COMSCN);  	// comscn
  sendData(0x01);

  sendCMD(OSCON);  		// oscon

  sendCMD(SLPOUT);  	// sleep out

  sendCMD(VOLCTR);  	// electronic volume, this is kinda contrast/brightness
  sendData(0x00);       // this might be different for individual LCDs
  sendData(0x03);    

  sendCMD(PWRCTR);  	// power ctrl
  sendData(0x0f);      	// everything on, no external reference resistors
  delay_ms(100);

  sendCMD(DISINV);  	// display mode

  sendCMD(DATCTL);  	// datctl
  sendData(0b00000011);
  sendData(0);
  sendData(0x01);
  sendData(0);

  sendCMD(RGBSET8);   	// setup color lookup table
    // color table

	sendData(0);
    sendData(2);
    sendData(4);
    sendData(6);
    sendData(8);
    sendData(10);
    sendData(12);
    sendData(15);
    // GREEN
    sendData(0);
    sendData(2);
    sendData(4);
    sendData(6);
    sendData(8);
    sendData(10);
    sendData(12);
    sendData(15);
    //BLUE
    sendData(0);
    sendData(4);
    sendData(9);
    sendData(15);

    sendCMD(NOP);
    lcd_clear(0xff,0,0,131,131);
    sendCMD(DISON);    // display on

   //for (i = 0; i < 150; i++){   // this loop adjusts the contrast, change the number of iterations to get
   //   sendCMD(VOLUP);           // desired contrast.  This might be different for individual LCDs
   //   delay_ms(2);
   //}
}

void draw_color_bar()
{
  lcd_clear(RED,0,0,131,33);
  lcd_clear(GREEN,0,34,131,66);
  lcd_clear(BLUE,0,67,131,99);
  lcd_clear(WHITE,0,100,131,131);
}

void draw_text_line(unsigned char fcolor, unsigned char bcolor,unsigned char x, unsigned char y,char c)
{
  unsigned int i;
  lcd_set_box(x,y,x,y+7);
  sendCMD(RAMWR);
  for(i=0;i<8;i++)
  {
    if (1<<i & c)
      sendData(fcolor);
    else
      sendData(bcolor);
  }
}

void lcd_draw_text(unsigned char fcolor, unsigned char bcolor, unsigned char x, unsigned char y,char *text)
{
  unsigned char c;
  unsigned char t;
  unsigned int i;
  unsigned int j;
  while(*text != 0)
   {
     t = *text;
     i = t - 32;
     i = i * 5;
     for(j = i; j < i+5; j++)
     {
       c = pgm_read_byte(&font[j]);
       draw_text_line(fcolor, bcolor, x++, y, c);
     }
     draw_text_line(fcolor, bcolor, x++, y, 0);
     text++;
   }
}

//  ************************************************************************************************* 
//      
//  Draws a line in the specified color from (x0,y0) to (x1,y1)  
//       
// 
//  Note:  good write-up on this algorithm in Wikipedia (search for Bresenham's line algorithm) 
//    see lcd.h for some sample color settings   
// 
//  Authors:   Dr. Leonard McMillan, Associate Professor UNC 
//      Jack Bresenham IBM, Winthrop University (Father of this algorithm, 1962) 
// 
//      Note: taken verbatim from Professor McMillan's presentation:  
//            http://www.cs.unc.edu/~mcmillan/comp136/Lecture6/Lines.html 
// 
//  *************************************************************************************************  
 
void LCD_Line(unsigned char x0, unsigned char y0, unsigned char x1, unsigned char y1, uint8_t color) { 
 
  int dy = y1 - y0; 
  int dx = x1 - x0; 
  int stepx, stepy; 
	if (dy < 0) { dy = -dy;  stepy = -1; } else { stepy = 1; } 
  if (dx < 0) { dx = -dx;  stepx = -1; } else { stepx = 1; } 
 
  dy <<= 1;        // dy is now 2*dy 
  dx <<= 1;        // dx is now 2*dx 
 
 
  LCD_Pixel(x0, y0, color);
  LCD_Pixel(x0, y0, color);  
 
  if (dx > dy) { 
 		int fraction = dy - (dx >> 1);  // same as 2*dy - dx 
    while (x0 != x1) { 
 	    if (fraction >= 0) { 
 				y0 += stepy; 
        fraction -= dx;    // same as fraction -= 2*dx 
 			} 
 			x0 += stepx; 
 
      fraction += dy;    // same as fraction -= 2*dy 
      LCD_Pixel(x0, y0, color);
		LCD_Pixel(x0, y0, color);	  
    } 
 	} 
 	else { 
 		int fraction = dx - (dy >> 1); 
    while (y0 != y1) { 
 			if (fraction >= 0) { 
 				x0 += stepx; 
 				fraction -= dy; 
      } 
      y0 += stepy; 
 			fraction += dx; 
      LCD_Pixel(x0, y0, color); 
	  LCD_Pixel(x0, y0, color); 
    } 
 	} 
 
} 
 
//  ***************************************************************************************** 
//  
//  Draws a rectangle in the specified color from (x1,y1) to (x2,y2) 
//  Rectangle can be filled with a color if desired  
// 
//    Author:  James P Lynch      July 7, 2007 
//  *****************************************************************************************  
 
 
 
void LCD_Box(unsigned char x0, unsigned char y0, unsigned char x1, unsigned char y1, unsigned char fill, uint8_t color) { 
  unsigned char   xmin, xmax, ymin, ymax; 
	int   i; 
 
 // check if the rectangle is to be filled 
  if (fill == 1) { 
 
    // best way to create a filled rectangle is to define a drawing box 
    // and loop two pixels at a time 
    // calculate the min and max for x and y directions 
      xmin = (x0 <= x1) ? x0 : x1;  
 		xmax = (x0 > x1) ? x0 : x1; 
		ymin = (y0 <= y1) ? y0 : y1; 
		ymax = (y0 > y1) ? y0 : y1; 
 
		// specify the controller drawing box according to those limits 
		// Row address set  (command 0x2B) 
		sendCMD(PASET); 
		sendData(xmin); 
		sendData( xmax); 
		// Column address set  (command 0x2A) 
		sendCMD(CASET); 
		sendData(ymin); 
		sendData(ymax); 
 
		// WRITE MEMORY 
		sendCMD(RAMWR); 
                
               
		// loop on total number of pixels / 2 
		for (i = 0; i < ((((xmax - xmin + 1) * (ymax - ymin + 1))/1 )+1 ); i++) { 
                  
  		  // use the color value to output three data bytes covering two pixels 
                 // sendData((color >> 4) & 0xFF ); 
  		 // sendData(((color & 0xF) << 4) | ((color >> 8) & 0xF) ); 
 		 // sendData( color & 0xFF); 
                   CS0
  		  /*SendLcd_color((color >> 4) & 0xFF ); 
  		  SendLcd_color(((color & 0xF) << 4) | ((color >> 8) & 0xF) ); 
 		  SendLcd_color(color & 0xFF);*/
		  uint8_t color8=0;
		  color8|=(color>>9);
		  color8<<=3;
		  color8|=(color>>5)&0b00000111;
		  color8<<=2;
		  color8|=(color>>2)&0b00000011;
		sendData( color8 );
		  
                   CS1
		} 
              
                                  
           sendCMD(NOP); 
	} 
    else { 

  // best way to draw un unfilled rectangle is to draw four lines 
 	 	LCD_Line(x0, y0, x1, y0, color); 
 	 	LCD_Line(x0, y1, x1, y1, color); 
 	 	LCD_Line(x0, y0, x0, y1, color); 
 	 	LCD_Line(x1, y0, x1, y1, color); 
  }
}

//  ************************************************************************************* 
//          LCD_Circle.c 
//  
//  Draws a line in the specified color at center (x0,y0) with radius  
//       
// 
//  Author:    Jack Bresenham IBM, Winthrop University (Father of this algorithm, 1962) 
// 
//         Note: taken verbatim Wikipedia article on Bresenham's line algorithm   
//          http://www.wikipedia.org 
// 
//  *************************************************************************************  
 
 
void LCD_Circle(unsigned char x0, unsigned char y0, unsigned char radius, uint8_t color) 
{ 
 
  int f = 1 - radius; 
 
  int ddF_x = 0; 
 
  int ddF_y = -2 * radius; 
  unsigned char x = 0; 
 
  unsigned char y = radius; 
 
  
 
  LCD_Pixel(x0, y0 + radius, color); 
  LCD_Pixel(x0, y0 - radius, color); 
 
  LCD_Pixel(x0 + radius, y0, color); 
 
  LCD_Pixel(x0 - radius, y0, color); 
  
 
  while (x < y) { 
 
  if (f >= 0) { 
 
   y--; 
   ddF_y += 2; 
 
   f += ddF_y; 
 
  } 
 
  x++; 
  ddF_x += 2; 
 
    f += ddF_x + 1;     
    LCD_Pixel(x0 + x, y0 + y, color); 
 
    LCD_Pixel(x0 - x, y0 + y, color); 
    LCD_Pixel(x0 + x, y0 - y, color); 
 
    LCD_Pixel(x0 - x, y0 - y, color); 
 
    LCD_Pixel(x0 + y, y0 + x, color); 
 
    LCD_Pixel(x0 - y, y0 + x, color); 
    LCD_Pixel(x0 + y, y0 - x, color); 
 
    LCD_Pixel(x0 - y, y0 - x, color); 

 } 
}
/************************************************************
MAIN
************************************************************/

int main (void) {

   lcd_init();
   delay_ms(200);

while(1)
  {
   draw_color_bar();

   delay_ms(2000);
   lcd_clear(BLACK,0,0,131,131);
   
   lcd_draw_text(YELLOW, BLACK, 7, 5, " TEST LCD6610 ");
   lcd_draw_text(CYAN, BLACK, 7, 15, " AND-TECH.PL");
   lcd_draw_text(RED, BLACK, 7, 30, " TEST RED COLOR ");
   lcd_draw_text(GREEN, BLACK, 7, 45, " TEST GREEN COLOR ");
   lcd_draw_text(BLUE, BLACK, 7, 60, " TEST BLUE COLOR ");
   lcd_draw_text(WHITE, BLACK, 7, 75, " TEST WHITE COLOR ");
   lcd_draw_text(MAGENTA, BLACK, 7, 90, " TEST MAGENTA COLOR ");
   lcd_draw_text(CYAN, BLACK, 7, 105, " TEST CYAN COLOR ");
   lcd_draw_text(YELLOW, BLACK, 7, 120, " TEST YELLOW COLOR ");

   delay_ms(2000);
   lcd_clear(BLACK,0,0,131,131);

   LCD_Line(0,0,131,131,WHITE);
   LCD_Line(131,0,0,131,GREEN);
   LCD_Circle(65,65,10,YELLOW);
   LCD_Box(10,10,120,120,0,BLUE);
   delay_ms(2000);
   lcd_clear(BLACK,0,0,131,131);
  }
}


